// 
// このコードは、https://demo.mescius.jp/diodocs/pdf のサンプルの一部として提供されています。
// © MESCIUS inc. All rights reserved.
// 
using System;
using System.IO;
using System.Drawing;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Pdf.AcroForms;
using GrapeCity.Documents.Text;
using GrapeCity.Documents.Drawing;
using System.Security.Cryptography.X509Certificates;
using GCTEXT = GrapeCity.Documents.Text;
using GCDRAW = GrapeCity.Documents.Drawing;

namespace DsPdfWeb.Demos
{
    // このサンプルは、SignatureField と署名画像を使用して、PDF を作成し
    // .pfx ファイルで署名する方法を示します。
    // また、サンプルは署名されたファイルを別の GcPdfDocument インスタンスに読み込み、
    // 署名を検証します。
    // このサンプルは SignDoc サンプルと同じですが、署名を表す画像を追加します。
    public class VisualSignature
    {
        public int CreatePDF(Stream stream)
        {
            var doc = new GcPdfDocument();
            var page = doc.NewPage();
            var tf = new TextFormat() { Font = Common.Util.getFont(), FontSize = 14 };
            page.Graphics.DrawString("デジタル署名のサンプルです\n" +
                "このPDFは表示している印影画像に署名されています。",
                tf, new PointF(72, 72));
            page.Graphics.DrawString(
                "注意：署名に使用している証明書、印影はテスト用です。\nブラウザのビルトインビューワによっては署名が表示されない場合があります。",
                new TextFormat(tf) { FontSize = 10 }, new PointF(72, 72 * 2));

            // テスト証明書を初期化します。
            var pfxPath = Path.Combine("Resources", "Misc", "GcPdfTest.pfx");
            var cert = new X509Certificate2(File.ReadAllBytes(pfxPath), "qq",
                X509KeyStorageFlags.MachineKeySet | X509KeyStorageFlags.PersistKeySet | X509KeyStorageFlags.Exportable);
            var sp = new SignatureProperties()
            {
                SignatureBuilder = new Pkcs7SignatureBuilder()
                {
                    CertificateChain = new X509Certificate2[] { cert }
                },
                Location = "DioDocs for PDF サンプル",
                SignerName = "株式会社ディオドック",
            };

            // 署名を表す画像を追加します。
            sp.SignatureAppearance.Image = GCDRAW.Image.FromFile(Path.Combine("Resources", "ImagesBis", "kaku_in.png"));
            sp.SignatureAppearance.CaptionImageRelation = GrapeCity.Documents.Pdf.Annotations.CaptionImageRelation.ImageOnly;

            // 署名を保持する署名フィールドを初期化します。
            var sf = new SignatureField();
            sf.Widget.Rect = new RectangleF(72 * 2, 72 * 2.5f, 72 * 2, 72 * 2);
            sf.Widget.Page = page;
            sf.Widget.BackColor = Color.LightSeaGreen;
            sf.Widget.DefaultAppearance.Font = Common.Util.getFont();
            sf.Widget.ButtonAppearance.Caption = $"Signer: {sp.SignerName}\r\nLocation: {sp.Location}";
            // 文書に署名フィールドを追加します。
            doc.AcroForm.Fields.Add(sf);
            // 署名フィールドと署名プロパティを結びつけます。
            sp.SignatureField = sf;

            // 署名して文書を保存します。
            // 注意：
            // - 署名と保存は一連の操作であり、２つは分離できません。
            // - Sign() メソッドに渡されたストリームは読み込み可能でなければなりません。
            doc.Sign(sp, stream);

            // ストリームを巻き戻して、作成した文書を別の GcPdfDocument
            // オブジェクトに読み込み、署名を確認します。
            stream.Seek(0, SeekOrigin.Begin);
            var doc2 = new GcPdfDocument();
            doc2.Load(stream);
            SignatureField sf2 = (SignatureField)doc2.AcroForm.Fields[0];
            if (!sf2.Value.VerifySignatureValue())
                throw new Exception("署名の検証に失敗しました。");

            // 終了（生成および署名された文書はすでに 'stream' に保存されています）。
            return doc.Pages.Count;
        }
    }
}
